// File Description
/// \file Strand.h
/// \brief Defines the Strand enum.
//
// Author: Derek Barnett

#ifndef STRAND_H
#define STRAND_H

#include "pbbam/Config.h"

#include <pbcopper/data/Strand.h>

namespace PacBio {
namespace BAM {

using Strand PBBAM_DEPRECATED = PacBio::Data::Strand;

}  // namespace BAM
}  // namespace PacBio

#endif  // STRAND_H
