#ifndef PBBAM_UNUSED_H
#define PBBAM_UNUSED_H

namespace PacBio {
namespace BAM {

template <typename T>
void UNUSED(const T&)
{
}

}  // namespace BAM
}  // namespace PacBio

#endif  // PBBAM_UNUSED_H
