// File Description
/// \file IFastqWriter.cpp
/// \brief Implements the IFastqWriter class.
//
// Author: Derek Barnett

#include "PbbamInternalConfig.h"

#include "pbbam/IFastqWriter.h"

namespace PacBio {
namespace BAM {

IFastqWriter::IFastqWriter() = default;

IFastqWriter::~IFastqWriter() = default;

}  // namespace BAM
}  // namespace PacBio
