// File Description
/// \file IRecordWriter.cpp
/// \brief Implements the IRecordWriter class.
//
// Author: Derek Barnett

#include "PbbamInternalConfig.h"

#include "pbbam/IRecordWriter.h"

namespace PacBio {
namespace BAM {

IRecordWriter::IRecordWriter() = default;

IRecordWriter::~IRecordWriter() = default;

}  // namespace BAM
}  // namespace PacBio
